             .title        "NES-QJ Reader Plugin"

;04/12/06
;Re-written by BootGod, original by KH
;Original version would only dump one game from cart because WRAM was
;not being enabled so the write to make it swap games was doing nothing.
             
;vectors for standard system calls
send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch

;variables
readaddr:    .equ 00e0h
reada_lo:    .equ 00e0h
reada_hi:    .equ 00e1h
numbanks:    .equ 00e2h
currbank:    .equ 00e3h

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "NES-QJ (Super Spike V'Ball "
             .db "/ Nintendo World Cup)"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 256 >> 4,0 >> 4
             .db 256 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             lda #04h		;send mirroring
             jsr send_byte

             lda #80h		;enable WRAM
             sta 0a001h
             
             lda #0h		;set to game 1
             sta 06000h
             lda #00h
             jsr send_byte
             lda #04h
             jsr send_byte
             lda #01h
             jsr send_byte
             jsr dump_prg
             lda #01h		;set to game 2
             sta 06000h
             jsr dump_prg

             lda #0h		;set to game 1
             sta 06000h
             lda #00h
             jsr send_byte
             lda #04h
             jsr send_byte
             lda #02h
             jsr send_byte
             jsr dump_chr
             lda #01h		;set to game 2
             sta 06000h
             jsr dump_chr
             
             lda #0		;all done
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts
             
dump_prg:    lda #0
             sta currbank
             lda #16		;16 * 8K = 128K
             sta numbanks
             
send_plp:    lda #6		;swap in desired bank
             sta 08000h
             lda currbank
             sta 08001h
             inc currbank
             lda #0		;set read address to $8000
             sta reada_lo
             lda #080h
             sta reada_hi
             ldx #020h		;set counters for loop
             ldy #0

send_plp2:   lda (readaddr),y	;send 8K bank
             jsr send_byte
             iny
             bne send_plp2
             inc reada_hi
             jsr baton
             dex
             bne send_plp2
             dec numbanks
             bne send_plp
             
             rts

dump_chr:    lda #0
             sta currbank
             lda #128		;128 * 1K = 128K
             sta numbanks

send_plp3:   lda #082h		;swap in desired bank
             sta 08000h
             lda currbank
             sta 08001h
             lda #00h		;set up ppu
             jsr wr_ppu
             lda 2007h
             ldy #0		;set counters for loop
             ldx #4

send_plp4:   lda 2007h		;send 1K bank
             jsr send_byte
             iny
             bne send_plp4
             jsr baton
             dex
             bne send_plp4
             inc currbank
             dec numbanks
             bne send_plp3
             
             rts

             .fill 0800h-*,0ffh	;fill rest to get 1K of data
             .end
